
/* Chrysalide - Outil d'analyse de fichiers binaires
 * bufferview.c - prototypes pour l'équivalent Python du fichier "gtkext/gtkbufferview.c"
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "bufferview.h"


#include <pygobject.h>


#include <gtkext/gtkbufferview.h>


#include "viewpanel.h"
#include "../helpers.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_buffer_view_type(void)
{
    static PyMethodDef py_buffer_view_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_buffer_view_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_buffer_view_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.gtkext.Bufferview",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide buffer view.",

        .tp_methods     = py_buffer_view_methods,
        .tp_getset      = py_buffer_view_getseters

    };

    return &py_buffer_view_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.gtkext.Bufferview'.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_buffer_view(PyObject *module)
{
    PyTypeObject *py_buffer_view_type;      /* Type Python 'Bufferview'    */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_buffer_view_type = get_python_buffer_view_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, GTK_TYPE_BUFFER_VIEW, py_buffer_view_type, get_python_view_panel_type()))
        return false;

    return true;

}
