
/* Chrysalide - Outil d'analyse de fichiers binaires
 * array.c - manipulations de tableaux propres à Dalvik
 *
 * Copyright (C) 2010 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "array.h"


#include "../../expression-int.h"



/* Définition d'une expression de mesure de tableau (instance) */
struct _GDalvikALength
{
    GDecExpression parent;                  /* A laisser en premier        */

    GDecExpression *array;                  /* Elément auquel accéder      */

};


/* Définition d'une expression de mesure de tableau (classe) */
struct _GDalvikALengthClass
{
    GDecExpressionClass parent;             /* A laisser en premier        */

};



/* Initialise la classe des expressions de mesure de tableau. */
static void g_dalvik_alength_class_init(GDalvikALengthClass *);

/* Initialise une instance d'expression de mesure de tableau. */
static void g_dalvik_alength_init(GDalvikALength *);

/* Imprime pour l'écran un version humaine d'une expression. */
static void g_dalvik_alength_print(const GDalvikALength *, GCodeBuffer *, GBufferLine *, GLangOutput *);



/* Indique le type défini pour une expression de mesure de tableau. */
G_DEFINE_TYPE(GDalvikALength, g_dalvik_alength, G_TYPE_DEC_EXPRESSION);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des expressions de mesure de tableau.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_alength_class_init(GDalvikALengthClass *klass)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : expr = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise une instance d'expression de mesure de tableau.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_alength_init(GDalvikALength *expr)
{
    GDecInstruction *instr;                 /* Autre version de l'objet    */

    instr = G_DEC_INSTRUCTION(expr);

    instr->print = (dec_instr_print_fc)g_dalvik_alength_print;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : array = tableau dont on doit mesurer la taille.              *
*                                                                             *
*  Description : Construit une expression à partir d'une mesure de tableau.   *
*                                                                             *
*  Retour      : Expression mise en place.                                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *g_dalvik_alength_new(GDecExpression *array)
{
    GDalvikALength *result;                 /* Expression à retourner      */

    result = g_object_new(G_TYPE_DALVIK_ALENGTH, NULL);

    result->array = array;

    return G_DEC_INSTRUCTION(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : expr   = expression à transcrire en version humaine.         *
*                buffer = tampon où doit se réaliser l'insertion.             *
*                line   = ligne d'impression prête à emploi ou NULL.          *
*                output = langage de programmation de sortie.                 *
*                                                                             *
*  Description : Imprime pour l'écran un version humaine d'une expression.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_alength_print(const GDalvikALength *expr, GCodeBuffer *buffer, GBufferLine *line, GLangOutput *output)
{
    g_dec_instruction_print(G_DEC_INSTRUCTION(expr->array),
                            buffer, line, output);

    g_buffer_line_append_text(line, BLC_ASSEMBLY, ".", 1, RTT_RAW, NULL);

    g_buffer_line_append_text(line, BLC_ASSEMBLY, "length", 6, RTT_RAW, NULL);

}
