
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ite.h - prototypes pour la représentation des branchements conditionnels
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DECOMP_INSTR_ITE_H
#define _DECOMP_INSTR_ITE_H


#include <glib-object.h>


#include "../expression.h"
#include "../instruction.h"
#include "../expr/cond.h"



#define G_TYPE_ITE_INSTRUCTION               g_ite_instruction_get_type()
#define G_ITE_INSTRUCTION(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_ite_instruction_get_type(), GITEInstruction))
#define G_IS_ITE_INSTRUCTION(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_ite_instruction_get_type()))
#define G_ITE_INSTRUCTION_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_ITE_INSTRUCTION, GITEInstructionClass))
#define G_IS_ITE_INSTRUCTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_ITE_INSTRUCTION))
#define G_ITE_INSTRUCTION_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_ITE_INSTRUCTION, GITEInstructionClass))



/* Définition d'un aiguillage du flux d'exécution (instance) */
typedef struct _GITEInstruction GITEInstruction;

/* Définition d'un aiguillage du flux d'exécution (classe) */
typedef struct _GITEInstructionClass GITEInstructionClass;


/* Indique le type défini pour un aiguillage du flux d'exécution. */
GType g_ite_instruction_get_type(void);

/* Exprime un aiguillage du flux en fonction d'une condition. */
GDecInstruction *g_ite_instruction_new(GCondExpression *, vmpa_t, vmpa_t);

/* Fournit la condition régissant la suite de l'exécution. */
GCondExpression *g_ite_instruction_get_condition(const GITEInstruction *);

/* Détermine le corps des différentes branches possibles. */
void g_ite_instruction_set_branches(GITEInstruction *, GDecInstruction *, GDecInstruction *);

/* Fournit le corps des différentes branches possibles. */
void g_ite_instruction_get_branches(const GITEInstruction *, GDecInstruction **, GDecInstruction **);



#endif  /* _DECOMP_INSTR_ITE_H */
