
/* Chrysalide - Outil d'analyse de fichiers binaires
 * core.c - chargement et le déchargement du tronc commun pour l'éditeur graphique
 *
 * Copyright (C) 2016 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "core.h"


#include "panels.h"


#include "../../core/params.h"
#include "../../glibext/linesegment.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : ref = espace de référencement global.                        *
*                                                                             *
*  Description : Charge les éléments graphiques de l'éditeur.                 *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool load_all_gui_components(GObject *ref)
{
    bool result;                            /* Bilan à retourner           */

    result = load_segment_rendering_parameters();

    result &= init_segment_content_hash_table();

    if (result)
        load_main_panels(ref);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : config = configuration globale à utiliser.                   *
*                                                                             *
*  Description : Finalise le chargement des éléments graphiques de l'éditeur. *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool complete_loading_of_all_gui_components(GGenConfig *config)
{
    bool result;                            /* Bilan à faire remonter      */

    result = browse_all_item_panels((handle_panel_item_fc)gtk_panel_item_apply_configuration, config);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Décharge les éléments graphiques de l'éditeur.               *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void unload_all_gui_components(void)
{
    exit_segment_content_hash_table();

}
